@@ -8,13 +8,13 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: math.tcl,v 1.17 2003/05/01 00:17:41 andreas_kupries Exp $
+# RCS: @(#) $Id: math.tcl,v 1.20 2005/10/06 05:16:37 andreas_kupries Exp $
 
 package require Tcl 8.2		;# uses [lindex $l end-$integer]
 
 namespace eval ::math {
 
-    variable version 1.2.2
+    variable version 1.2.3
 
     # misc.tcl
 
@@ -22,7 +22,7 @@ namespace eval ::math {
     namespace export	max		mean		min
     namespace export	product		random		sigma
     namespace export	stats		sum
-    namespace export	expectDouble
+    namespace export	expectDouble    expectInteger
 
     # combinatorics.tcl
 
