@@ -8,14 +8,16 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id: md4.tcl,v 1.5 2003/05/01 00:17:41 andreas_kupries Exp $
+# $Id: md4.tcl,v 1.16 2005/02/24 03:25:49 patthoyts Exp $
 
 package require Tcl 8.2;                # tcl minimum version
 catch {package require md4c 1.0};       # tcllib critcl alternative
 
 namespace eval ::md4 {
-    variable version 1.0.0
-    variable rcsid {$Id: md4.tcl,v 1.5 2003/05/01 00:17:41 andreas_kupries Exp $}
+    variable version 1.0.3
+    variable rcsid {$Id: md4.tcl,v 1.16 2005/02/24 03:25:49 patthoyts Exp $}
+    variable accel
+    array set accel {critcl 0 cryptkit 0}
 
     namespace export md4 hmac MD4Init MD4Update MD4Final
 
@@ -32,31 +34,40 @@ namespace eval ::md4 {
 #
 proc ::md4::MD4Init {} {
     variable uid
+    variable accel
     set token [namespace current]::[incr uid]
-    upvar #0 $token tok
+    upvar #0 $token state
 
     # RFC1320:3.3 - Initialize MD4 state structure
-    array set tok \
+    array set state \
         [list \
-             A [expr 0x67452301] \
-             B [expr 0xefcdab89] \
-             C [expr 0x98badcfe] \
-             D [expr 0x10325476] \
+             A [expr {0x67452301}] \
+             B [expr {0xefcdab89}] \
+             C [expr {0x98badcfe}] \
+             D [expr {0x10325476}] \
              n 0 i "" ]
+    if {$accel(cryptkit)} {
+        cryptkit::cryptCreateContext state(ckctx) CRYPT_UNUSED CRYPT_ALGO_MD4
+    }
     return $token
 }
 
 proc ::md4::MD4Update {token data} {
-    variable $token
-    upvar 0 $token state
+    variable accel
+    upvar #0 $token state
 
-    if {[package provide md4c] != {}} {
+    if {$accel(critcl)} {
         if {[info exists state(md4c)]} {
             set state(md4c) [md4c $data $state(md4c)]
         } else {
             set state(md4c) [md4c $data]
         }
         return
+    } elseif {[info exists state(ckctx)]} {
+        if {[string length $data] > 0} {
+            cryptkit::cryptEncrypt $state(ckctx) $data
+        }
+        return
     }
 
     # Update the state values
@@ -75,13 +86,22 @@ proc ::md4::MD4Update {token data} {
 }
 
 proc ::md4::MD4Final {token} {
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
 
-    if {[package provide md4c] != {}} {
+    if {[info exists state(md4c)]} {
         set r $state(md4c)
         unset state
         return $r
+    } elseif {[info exists state(ckctx)]} {
+        cryptkit::cryptEncrypt $state(ckctx) ""
+        cryptkit::cryptGetAttributeString $state(ckctx) \
+            CRYPT_CTXINFO_HASHVALUE r 16
+        cryptkit::cryptDestroyContext $state(ckctx)
+        # If nothing was hashed, we get no r variable set!
+        if {[info exists r]} {
+            unset state
+            return $r
+        }
     }
 
     # RFC1320:3.1 - Padding
@@ -106,7 +126,7 @@ proc ::md4::MD4Final {token} {
     }
 
     # RFC1320:3.5 - Output
-    set r [binary format i4 [list $state(A) $state(B) $state(C) $state(D)]]
+    set r [bytes $state(A)][bytes $state(B)][bytes $state(C)][bytes $state(D)]
     unset state
     return $r
 }
@@ -143,6 +163,7 @@ proc ::md4::HMACInit {K} {
     MD4Update $tok $Ki;                 # initialize with the inner pad
     
     # preserve the Ko value for the final stage.
+    # FRINK: nocheck
     set [subst $tok](Ko) $Ko
 
     return $tok
@@ -154,6 +175,7 @@ proc ::md4::HMACUpdate {token data} {
 }
 
 proc ::md4::HMACFinal {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
@@ -255,19 +277,31 @@ set ::md4::MD4Hash_body {
         # Then perform the following additions. (That is, increment each
         # of the four registers by the value it had before this block
         # was started.)
-        set state(A) [expr {($A + $state(A)) & 0xFFFFFFFF}]
-        set state(B) [expr {($B + $state(B)) & 0xFFFFFFFF}]
-        set state(C) [expr {($C + $state(C)) & 0xFFFFFFFF}]
-        set state(D) [expr {($D + $state(D)) & 0xFFFFFFFF}]
+        incr state(A) $A
+        incr state(B) $B
+        incr state(C) $C
+        incr state(D) $D
     }
 
     return
 }
 
+proc ::md4::byte {n v} {expr {((0xFF << (8 * $n)) & $v) >> (8 * $n)}}
+proc ::md4::bytes {v} { 
+    #format %c%c%c%c [byte 0 $v] [byte 1 $v] [byte 2 $v] [byte 3 $v]
+    format %c%c%c%c \
+        [expr {0xFF & $v}] \
+        [expr {(0xFF00 & $v) >> 8}] \
+        [expr {(0xFF0000 & $v) >> 16}] \
+        [expr {((0xFF000000 & $v) >> 24) & 0xFF}]
+}
+
 # 32bit rotate-left
 proc ::md4::<<< {v n} {
-    set v [expr {(($v << $n) | (($v >> (32 - $n)) & (0x7FFFFFFF >> (31 - $n))))}]
-    return [expr {$v & 0xFFFFFFFF}]
+    return [expr {((($v << $n) \
+                        | (($v >> (32 - $n)) \
+                               & (0x7FFFFFFF >> (31 - $n))))) \
+                      & 0xFFFFFFFF}]
 }
 
 # Convert our <<< pseuodo-operator into a procedure call.
@@ -319,20 +353,54 @@ proc ::md4::MD4Hash {token msg} $::md4::MD4Hash_body
 # -------------------------------------------------------------------------
 
 if {[package provide Trf] != {}} {
-    interp alias {} ::md4::Hex {} ::hex -mode encode
+    interp alias {} ::md4::Hex {} ::hex -mode encode --
 } else {
     proc ::md4::Hex {data} {
-        set result {}
-        binary scan $data c* r
-        foreach c $r {
-            append result [format "%02X" [expr {$c & 0xff}]]
-        }
-        return $result
+        binary scan $data H* result
+        return [string toupper $result]
     }
 }
 
 # -------------------------------------------------------------------------
 
+# LoadAccelerator --
+#
+#	This package can make use of a number of compiled extensions to
+#	accelerate the digest computation. This procedure manages the
+#	use of these extensions within the package. During normal usage
+#	this should not be called, but the test package manipulates the
+#	list of enabled accelerators.
+#
+proc ::md4::LoadAccelerator {name} {
+    variable accel
+    set r 0
+    switch -exact -- $name {
+        critcl {
+            if {![catch {package require tcllibc}]
+                || ![catch {package require md4c}]} {
+                set r [expr {[info command ::md4::md4c] != {}}]
+            }
+        }
+        cryptkit {
+            if {![catch {package require cryptkit}]} {
+                set r [expr {![catch {cryptkit::cryptInit}]}]
+            }
+        }
+        #trf {
+        #    if {![catch {package require Trf}]} {
+        #        set r [expr {![catch {::md4 aa} msg]}]
+        #    }
+        #}
+        default {
+            return -code error "invalid accelerator package:\
+                must be one of [join [array names accel] {, }]"
+        }
+    }
+    set accel($name) $r
+}
+
+# -------------------------------------------------------------------------
+
 # Description:
 #  Pop the nth element off a list. Used in options processing.
 #
@@ -348,6 +416,7 @@ proc ::md4::Pop {varname {nth 0}} {
 # fileevent handler for chunked file hashing.
 #
 proc ::md4::Chunk {token channel {chunksize 4096}} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     
@@ -369,8 +438,9 @@ proc ::md4::md4 {args} {
             -file*     { set opts(-filename) [Pop args 1] }
             -channel   { set opts(-channel) [Pop args 1] }
             -chunksize { set opts(-chunksize) [Pop args 1] }
-            --         { Pop args ; break }
             default {
+                if {[llength $args] == 1} { break }
+                if {[string compare $option "--"] == 0 } { Pop args; break }
                 set err [join [lsort [array names opts]] ", "]
                 return -code error "bad option $option:\
                     must be one of $err"
@@ -397,6 +467,7 @@ proc ::md4::md4 {args} {
     } else {
 
         set tok [MD4Init]
+        # FRINK: nocheck
         set [subst $tok](reading) 1
         fileevent $opts(-channel) readable \
             [list [namespace origin Chunk] \
@@ -419,7 +490,7 @@ proc ::md4::md4 {args} {
 # -------------------------------------------------------------------------
 
 proc ::md4::hmac {args} {
-    array set opts {-hex 0 -filename {} -channel {} -chunksize 4096 -key {}}
+    array set opts {-hex 0 -filename {} -channel {} -chunksize 4096}
     while {[string match -* [set option [lindex $args 0]]]} {
         switch -glob -- $option {
             -key       { set opts(-key) [Pop args 1] }
@@ -427,8 +498,9 @@ proc ::md4::hmac {args} {
             -file*     { set opts(-filename) [Pop args 1] }
             -channel   { set opts(-channel) [Pop args 1] }
             -chunksize { set opts(-chunksize) [Pop args 1] }
-            --         { Pop args ; break }
             default {
+                if {[llength $args] == 1} { break }
+                if {[string compare $option "--"] == 0 } { Pop args; break }
                 set err [join [lsort [array names opts]] ", "]
                 return -code error "bad option $option:\
                     must be one of $err"
@@ -437,7 +509,7 @@ proc ::md4::hmac {args} {
         Pop args
     }
 
-    if {$opts(-key) == {}} {
+    if {![info exists opts(-key)]} {
         return -code error "wrong # args:\
             should be \"hmac ?-hex? -key key -filename file | string\""
     }
@@ -460,6 +532,7 @@ proc ::md4::hmac {args} {
     } else {
 
         set tok [HMACInit $opts(-key)]
+        # FRINK: nocheck
         set [subst $tok](reading) 1
         fileevent $opts(-channel) readable \
             [list [namespace origin Chunk] \
@@ -481,6 +554,11 @@ proc ::md4::hmac {args} {
 
 # -------------------------------------------------------------------------
 
+# Try and load a compiled extension to help.
+namespace eval ::md4 {
+    foreach e {critcl cryptkit} { if {[LoadAccelerator $e]} { break } }
+}
+
 package provide md4 $::md4::version
 
 # -------------------------------------------------------------------------
